Imports System.IO.Ports

Public Class frmMain

   Dim txBuf As Byte()                    'This is our serial port output buffer
   Dim rxBuf As Byte()                    'This is our serial port input buffer
   Dim cmdBuf As Byte()                   'This is our Command Byte array
   Dim txString As Char()                 'Char array for our string output
   Dim rxString As Char()                 'Char array for our string input
   Dim offset As Integer = 0              'Index into the output buffer
   Dim count As Integer = 0               'Number of bytes we are going to send
   Dim commPort As String
   Dim connectedFlag As Boolean = False   'Flag to indicate connection state

   ' Enumerate the commads as an index in to the command buffer (cmdBuf)
   Const CmdTxN = 0
   Const CmdRxN = 1
   Const CmdTxS = 2
   Const CmdRxS = 3
   Const cmdTrm = 4

   Private Sub frmMain_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
      ReDim txBuf(255)     ' Make room in our output buffer for some data
      ReDim rxBuf(255)     ' Make room in our input buffer for some data
      ReDim cmdBuf(5)      ' Make room in our command buffer for some data
      ReDim rxString(255)  ' Setup our character receive array

      ' Initialize controls
      cmdDnload.Enabled = False
      cmdUpload.Enabled = False

      optDataType0.Checked = True
      optDataType1.Checked = False

      scrValueChange()
      txtStringData.Clear()

      ' Load the commands in to the command buffer
      cmdBuf(0) = &HC0
      cmdBuf(1) = &HC1
      cmdBuf(2) = &HC2
      cmdBuf(3) = &HC3
      cmdBuf(4) = 0

      ' Setup some default comm port attributes
      With MSComm1
         .BaudRate = 9600
         .Parity = Parity.None
         .DataBits = 8
         .StopBits = StopBits.One
         .Handshake = Handshake.RequestToSend    ' Use hardware flow control
         .ReadTimeout = 1000
         .RtsEnable = True
      End With

      Get_COMPorts()

   End Sub

   Private Sub Get_COMPorts()
      Dim ports As String()

      ' Get a list of serial port names.
      ports = SerialPort.GetPortNames()

      ' Display each port name to the listbox.
      Dim port As String
      For Each port In ports
         lstCommPorts.Items.Add(port)
      Next port

      lblSelectCommPort.Visible = True
      lstCommPorts.Visible = True
      btnSelectCommPort.Visible = True

   End Sub

   Private Sub lstCommPorts_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles lstCommPorts.SelectedIndexChanged

      ' Get the comm port name
      commPort = lstCommPorts.SelectedItem.ToString
      ' Enable the connect button
      btnSelectCommPort.Enabled = True

   End Sub

   Private Sub btnSelectCommPort_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSelectCommPort.Click

      If Not connectedFlag Then
         MSComm1.PortName = commPort
         Try
            MSComm1.Open()
            Me.Text = "PC-to-Stamp Comm [Connected]"
         Catch ex As Exception
            MessageBox.Show("ERROR: Could not connect.", "PC-to-Stamp Comm", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
         End Try
         connectedFlag = True
         btnSelectCommPort.Text = "Disconnect"
         mnuFileExit.Enabled = False
         btnExit.Enabled = False
         EnableButtons()
      Else
         MSComm1.Close()
         connectedFlag = False
         Me.Text = "PC-to-Stamp Comm"
         btnSelectCommPort.Text = "Connect"
         mnuFileExit.Enabled = True
         btnExit.Enabled = True
         DisableButtons()
      End If

   End Sub

   Private Sub cmdDnload_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdDnload.Click
      Dim chrPos As Integer

      If Not MSComm1.IsOpen Then
         MessageBox.Show("Comm Port Not Open...", "Comm Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
         Exit Sub
      End If

      ' Stop other transfers until this one completes
      DisableButtons()

      If optDataType0.Checked = True Then
         ' Send number command
         MSComm1.Write(cmdBuf, CmdTxN, 1)
         ' Send word value; low byte first
         txBuf(0) = scrValue.Value Mod 256
         txBuf(1) = scrValue.Value \ 256
         MSComm1.Write(txBuf, 0, 2)
         FlushTxBuf()
      Else
         ' Place string into buffer
         txString = txtStringData.Text.ToCharArray
         ' Send string command
         MSComm1.Write(cmdBuf, CmdTxS, 1)
         FlushTxBuf()
         ' Send one character at a time
         For chrPos = 0 To txtStringData.TextLength - 1
            MSComm1.Write(txString, chrPos, 1)
            FlushTxBuf()
         Next
         ' Send terminating character
         MSComm1.Write(cmdBuf, cmdTrm, 1)
         FlushTxBuf()
      End If

      ' Allow transfers
      EnableButtons()

   End Sub

   Private Sub cmdUpload_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdUpload.Click
      Dim strLen, x As Integer

      If Not MSComm1.IsOpen Then
         MessageBox.Show("Comm Port Not Open...", "Comm Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
         Exit Sub
      End If

      ' Make sure input buffer is clear
      FlushRxBuf()

      ' Stop other transfers until this one completes
      DisableButtons()

      If optDataType0.Checked = True Then
         ' Send command
         MSComm1.Write(cmdBuf, CmdRxN, 1)
         FlushTxBuf()
         ' Wait for two-byte value
         Do
            ' Get 2 bytes
         Loop Until (MSComm1.BytesToRead = 2)
         ' Grab input buffer
         MSComm1.Read(rxBuf, 0, 2)
         ' Display it
         scrValue.Value = rxBuf(0) + (rxBuf(1) * 256)
         ' Update scroller
         scrValueChange()
         FlushRxBuf()
      Else
         For x = rxString.GetLowerBound(0) To rxString.GetUpperBound(0)
            rxString(x) = ""
         Next
         ' Clear text box
         txtStringData.Clear()
         ' Send string command
         MSComm1.Write(cmdBuf, CmdRxS, 1)
         FlushTxBuf()
         Do
            ' Wait for string length
         Loop Until (MSComm1.BytesToRead = 1)
         MSComm1.Read(rxBuf, 0, 1)
         ' Extract string length
         strLen = rxBuf(0)
         FlushRxBuf()
         ' Resend command to strart upload
         MSComm1.Write(cmdBuf, CmdRxS, 1)
         FlushTxBuf()
         Do
            ' Wait for string
         Loop Until (MSComm1.BytesToRead = strLen)
         MSComm1.Read(rxString, 0, strLen)
         ' Show it
         FlushRxBuf()
         txtStringData.Text = rxString
      End If

      ' Allow transfers
      EnableButtons()

   End Sub

   Private Sub FlushTxBuf()

      MSComm1.DiscardOutBuffer()

   End Sub

   Private Sub FlushRxBuf()

      MSComm1.DiscardInBuffer()

   End Sub

   Private Sub scrValueChange()

      lblValue.Text = scrValue.Value.ToString

   End Sub

   Private Sub scrValue_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles scrValue.Scroll

      lblValue.Text = scrValue.Value.ToString

   End Sub

   Private Sub DisableButtons()

      cmdDnload.Enabled = False
      cmdUpload.Enabled = False

   End Sub

   Private Sub EnableButtons()

      cmdDnload.Enabled = True
      cmdUpload.Enabled = True
      
   End Sub

   Private Sub mnuFileExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuFileExit.Click, btnExit.Click

      If mnuFileExit.Enabled Then
         Me.Close()
      Else
         MessageBox.Show("Data transfer in progress...", "Can not close", MessageBoxButtons.OK, MessageBoxIcon.Warning)
      End If

   End Sub

   Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing

      If MSComm1.IsOpen Then
         MSComm1.Close()
      End If

   End Sub

End Class
